<?php

namespace Drupal\drd\Plugin\Update;

use Drupal\Core\File\FileSystem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\drd\UpdatePluginInterface;

/**
 * Abstract DRD Update plugin to implement general functionality.
 */
abstract class Base extends PluginBase implements UpdatePluginInterface {

  /** @var FileSystem */
  protected $fs;

  /** @var string */
  protected $configFormParent;

  /** @var string */
  protected $condition;

  /** @var string */
  protected $workingDirectory;

  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fs = \Drupal::service('file_system');
  }

  /**
   * @inheritDoc
   */
  public function setConfigFormContext($parent, $condition) {
    $this->configFormParent = $parent;
    $this->condition = $condition;
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function getConfiguration() {
    return isset($this->configuration) ? $this->configuration : $this->defaultConfiguration();
  }

  /**
   * @inheritDoc
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function calculateDependencies() {
    return [];
  }

  /**
   * @inheritDoc
   */
  public function defaultConfiguration() {
    return [
      'scripts' => [
        'prepare' => '',
        'update' => '',
        'cleanup' => '',
      ],
    ];
  }

  /**
   * @inheritDoc
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $element = [];

    $element['scripts'] = [
      '#type' => 'details',
      '#title' => $this->t('Scripts'),
      '#open' => FALSE,
      '#weight' => 99,
    ];
    foreach (['prepare', 'update', 'cleanup'] as $script) {
      $element['scripts'][$script] = [
        '#type' => 'textfield',
        '#title' => $this->t('After :script', [':script' => $script]),
        '#default_value' => $this->configuration['scripts'][$script],
      ];
    }
    $element['#tree'] = TRUE;

    return $element;
  }

  /**
   * @inheritDoc
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    foreach (['prepare', 'update', 'cleanup'] as $script) {
      $value = $form_state->getValue([$this->pluginId, 'scripts', $script]);
      if (!empty($value) && !file_exists($value)) {
        $form_state->setError($form[$this->configFormParent][$this->pluginId]['scripts'][$script], $this->t('Script doesn\'t exist.'));
      }
    }
  }

  /**
   * @inheritDoc
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    foreach (['prepare', 'update', 'cleanup'] as $script) {
      $this->configuration['scripts'][$script] = $form_state->getValue([$this->pluginId, 'scripts', $script]);
    }
  }

  private function executeScript($type) {
    # TODO: Implement script execution
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function execute($releases) {
    $this->setWorkingDirectory()
      ->prepareWorkingDirectory()
      ->executeScript('prepare')
      ->updateWorkingDirectory($releases)
      ->executeScript('update')
      ->cleanup()
      ->executeScript('cleanup');
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function setWorkingDirectory() {
    if (!isset($this->workingDirectory)) {
      $this->workingDirectory = $this->fs->tempnam(file_directory_temp(), 'drd-update-');
    }
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function prepareWorkingDirectory() {
    $this->fs->unlink($this->workingDirectory);
    $this->fs->mkdir($this->workingDirectory);
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cleanup() {
    $this->fs->rmdir($this->workingDirectory);
  }

}
