<?php

namespace Drupal\drd;

use Drupal\Component\Plugin\FallbackPluginManagerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Manages discovery and instantiation of DRD Update plugins.
 */
class UpdateManager extends DefaultPluginManager implements UpdateManagerInterface, FallbackPluginManagerInterface {

  /** @var UpdatePluginInterface[] */
  private $plugins = [];

  /**
   * Constructs a new \Drupal\drd\UpdateManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/Update', $namespaces, $module_handler, 'Drupal\drd\UpdatePluginInterface', 'Drupal\drd\Annotation\Update');

    $this->alterInfo('drd_update');
    $this->setCacheBackend($cache_backend, 'drd_update_plugins');
  }

  /**
   * {@inheritdoc}
   */
  public function getFallbackPluginId($plugin_id, array $configuration = []) {
    return 'broken';
  }

  /**
   * @inheritDoc
   */
  public function buildGlobalForm(&$form, $form_state, $settings) {
    $settings += [
      'current' => '',
      'details' => [],
    ];
    $list = ['' => t('None')];
    foreach ($this->getDefinitions() as $def) {
      $list[$def['id']] = $def['admin_label'];
    }

    $form['drd_update_method'] = [
      '#type' => 'fieldset',
      '#title' => t('Update method'),
    ];
    $form['drd_update_method']['drd_upd_type'] = [
      '#type' => 'select',
      '#options' => $list,
      '#default_value' => $settings['current'],
    ];
    foreach ($this->getDefinitions() as $def) {
      /** @var UpdatePluginInterface $upd */
      $upd = $this->createInstance($def['id']);
      $upd->setConfiguration(isset($settings['details'][$def['id']]) ?
        $settings['details'][$def['id']] :
        []
      );

      $condition = ['select#edit-drd-upd-type' => ['value' => $def['id']]];
      $upd->setConfigFormContext('drd_update_method', $condition);
      $form['drd_update_method'][$def['id']] = [
        '#type' => 'container',
        '#states' => [
          'visible' => $condition,
        ],
      ];
      $form['drd_update_method'][$def['id']] += $upd->buildConfigurationForm($form, $form_state);
      $this->plugins[$def['id']] = $upd;
    }
  }

  /**
   * @inheritDoc
   */
  public function validateGlobalForm(&$form, $form_state, $key) {
    $current = $form_state->getValue('drd_upd_type');
    foreach ($this->plugins as $id => $plugin) {
      if ($id == $current) {
        $plugin->validateConfigurationForm($form, $form_state);
      }
    }
  }

  /**
   * @inheritDoc
   */
  public function globalFormValues($form, $form_state, $key) {
    $settings = [
      'current' => $form_state->getValue('drd_upd_type'),
      'details' => [],
    ];
    foreach ($this->plugins as $id => $plugin) {
      $plugin->submitConfigurationForm($form, $form_state);
      $settings['details'][$id] = $plugin->getConfiguration();
    }
    return $settings;
  }

}
